@@ -107,6 +107,7 @@ CLI_ARGUMENTS              = collections.OrderedDict((
                                       'metavar' : ('VARS', 'PROCS')})]),
 
     ('Export options', [
+        (('io',  'ids_only'),              {'action'  : 'store_true'}),
         (('f',   'format'),                {}),
         (('edf', 'date_format'),           {'default' : 'default'}),
         (('etf', 'time_format'),           {'default' : 'default'}),
@@ -143,6 +144,7 @@ CLI_ARGUMENTS              = collections.OrderedDict((
         (('V',  'version'),      {'action'  : 'version',
                                   'version' :
                                   '%(prog)s {}'.format(VERSION)}),
+        (('dn', 'drop_na_rows'), {'action' : 'store_true'}),
         (('d',  'dry_run'),      {'action' : 'store_true'}),
         (('nb', 'no_builtins'),  {'action' : 'store_true'}),
         (('nj', 'num_jobs'),     {'type'    : int,
@@ -162,8 +164,8 @@ CLI_DESCRIPTIONS = {
     'variable/data coding IDs.',
 
     'Export options' :
-    'Non-numeric columns are exported to the main output file by default, but\n'
-    'you can control this behaviour using one of the following options:\n'
+    'Non-numeric columns are exported to the main output file by default,\n'
+    'but you can control this behaviour using one of the following options:\n'
     '\n'
     ' - The --suppress_non_numerics option tells FUNPACK to only save\n'
     '   numeric columns to the main output file.\n'
@@ -326,6 +328,11 @@ CLI_ARGUMENT_HELP = {
     'table.',
 
     # Export options
+    'ids_only' :
+    'Do not output any data - instead, output a plain-text file which just '
+    'contains the subject IDs, one per line. When this option is used, all '
+    'cleaning and processing routines will be skipped, and no other output '
+    'files (with the exception of a log file) will be produced.',
     'format' :
     'Output file format (default: inferred from the output file suffix - one '
     'of "tsv", "csv", or "h5").',
@@ -381,6 +388,8 @@ CLI_ARGUMENT_HELP = {
     # Miscellaneous options
     'version'      : 'Print version and exit.',
     'dry_run'      : 'Print a summary of what would happen and exit.',
+    'drop_na_rows' : 'Drop rows which do not contain data for any column. '
+                     'will take place on both import and export.',
     'no_builtins'  : 'Do not use the built in variable or data coding tables.',
     'num_jobs'     : 'Maximum number of jobs to run in parallel. Set to 1 '
                      '(the default) to disable parallelisation.  Set to -1 '
@@ -515,6 +524,56 @@ def sanitiseArgs(argv):
     return argv
 
 
+def loadConfigFile(cfgfile, namespace=None):
+    """Loads arguments from the given configuration file, returning an
+    ``argparse.Namespace`` object.
+
+    :arg cfgfile:   Path to configuration file
+    :arg namespace: Existing ``argparse.Namespace`` object to merge
+                    arguments into.
+    """
+
+    argv = []
+
+    # If the specified config file does
+    # not exist, assume it is a reference
+    # to a built-in config file
+    if not op.exists(cfgfile):
+        if not cfgfile.endswith('.cfg'):
+            cfgfile = cfgfile + '.cfg'
+        moddir   = op.dirname(op.abspath(__file__))
+        cfgfile = op.join(moddir, 'configs', cfgfile)
+
+    log.debug('Loading arguments from configuration file %s', cfgfile)
+
+    with open(cfgfile, 'rt') as f:
+        lines = [line.strip() for line in f.readlines()]
+        lines = [line         for line in lines if line != '']
+        lines = [line         for line in lines if not line.startswith('#')]
+
+    for line in lines:
+        words        = list(shlex.split(line))
+        name, values = words[0], words[1:]
+
+        argv.append('--{}'.format(name))
+        argv.extend(values)
+
+    argv      = sanitiseArgs(argv)
+    parser    = makeParser(exclude=('outfile', 'infile'))
+    namespace = parser.parse_args(argv, namespace)
+
+    # a config file may "include" other
+    # config files - we parse them recursively,
+    # and merge the arguments in load order.
+    cfgfiles = namespace.config_file
+    if cfgfiles is not None and len(cfgfiles) > 0:
+        for cf in cfgfiles:
+            namespace.config_file = None
+            namespace             = loadConfigFile(cf, namespace)
+
+    return namespace
+
+
 def parseArgsWithConfigFile(argv=None):
     """Checks the command line arguments to see if one or more configuration
     files has been specified. If so, loads the arguments in the configuration
@@ -565,34 +624,37 @@ def parseArgs(argv=None, namespace=None):
               'specified!')
         sys.exit(1)
 
-    # make input/output files absolute
-    args.infile  = [op.realpath(f) for f in args.infile]
-    args.outfile =  op.realpath(args.outfile)
-
-    # assign default names to
-    # auxillary output files
-    outbase, outext = op.splitext(args.outfile)
-    auxfiles        = ['log', 'non_numerics', 'unknown_vars',
-                       'icd10_map', 'description', 'summary']
-    auxexts         = collections.defaultdict(lambda : '.txt',
-                                              non_numerics=outext)
-    for auxfile in auxfiles:
-        writeatt = 'write_{}'.format(auxfile)
-        fileatt  = '{}_file' .format(auxfile)
-        write    = getattr(args, writeatt)
-        filename = getattr(args, fileatt)
-
-        if write and filename is None:
-            setattr(args, fileatt, '{}_{}{}'.format(outbase,
-                                                    auxfile,
-                                                    auxexts[auxfile]))
-        else:
-            setattr(args, writeatt, filename is not None)
-
     # -1 implies max-parallel
     if   args.num_jobs <= -1: args.num_jobs = mp.cpu_count()
     elif args.num_jobs ==  0: args.num_jobs = 1
 
+    if args.noisy is None: args.noisy = 0
+    if args.quiet:         args.noisy = 0
+
+    _prepareInputAndOutputFiles(        args)
+    _prepareTableFiles(                 args)
+    _prepareAuxFileNames(               args)
+    _prepareSubjectAndVariableSelectors(args)
+    _prepareCategorySelectors(          args)
+    _prepareColumnSelectors(            args)
+    _prepareVisitSelectors(             args)
+    _prepareCleaningSelectors(          args)
+    _prepareIDsOnly(                    args)
+
+    return args, argv
+
+
+def _prepareInputAndOutputFiles(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    the primary input and output files.
+
+    :arg args: ``argparse.Namespace`` object.
+    """
+
+    # make input/output files absolute
+    args.infile  = [op.realpath(f) for f in args.infile]
+    args.outfile =  op.realpath(args.outfile)
+
     # the importing.loadData function accepts
     # either a single encoding, or one encoding
     # for each data file.
@@ -622,7 +684,7 @@ def parseArgs(argv=None, namespace=None):
         args.var_format = []
 
     var_format = {}
-    for i, (v, n) in enumerate(args.var_format):
+    for v, n in args.var_format:
 
         # Formatters should be set on integer
         # variable IDs. But we allow non-integers
@@ -636,6 +698,83 @@ def parseArgs(argv=None, namespace=None):
 
     args.var_format = var_format
 
+
+def _prepareTableFiles(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    the input table files.
+
+    :arg args: ``argparse.Namespace`` object.
+    """
+
+    # The variable_file, datacoding_file, type_file,
+    # processing_file, and category_file options
+    # can be specified relative to funpackdir
+    configdir = op.join(op.dirname(__file__), 'configs')
+
+    def fixPath(f):
+
+        if f is None:
+            return f
+
+        options = [
+            f,
+            op.join(configdir, f),
+            op.join(configdir, f.replace('.', op.sep) + '.tsv')
+        ]
+
+        for o in options:
+            if op.exists(o):
+                return o
+
+        # if the fixed version does not
+        # exist, allow processing to
+        # continue - it will fail later on.
+        return f
+
+    if args.variable_file is not None:
+        args.variable_file = [fixPath(f) for f in args.variable_file]
+    if args.datacoding_file is not None:
+        args.datacoding_file = [fixPath(f) for f in args.datacoding_file]
+
+    args.type_file       = fixPath(args.type_file)
+    args.processing_file = fixPath(args.processing_file)
+    args.category_file   = fixPath(args.category_file)
+
+
+def _prepareAuxFileNames(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    the auxillary output files.
+
+    :arg args: ``argparse.Namespace`` object.
+    """
+    # assign default names to
+    # auxillary output files
+    outbase, outext = op.splitext(args.outfile)
+    auxfiles        = ['log', 'non_numerics', 'unknown_vars',
+                       'icd10_map', 'description', 'summary']
+    auxexts         = collections.defaultdict(lambda : '.txt',
+                                              non_numerics=outext)
+    for auxfile in auxfiles:
+        writeatt = 'write_{}'.format(auxfile)
+        fileatt  = '{}_file' .format(auxfile)
+        write    = getattr(args, writeatt)
+        filename = getattr(args, fileatt)
+
+        if write and filename is None:
+            setattr(args, fileatt, '{}_{}{}'.format(outbase,
+                                                    auxfile,
+                                                    auxexts[auxfile]))
+        else:
+            setattr(args, writeatt, filename is not None)
+
+
+def _prepareSubjectAndVariableSelectors(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    the subject and variable selection options.
+
+    :arg args: ``argparse.Namespace`` object.
+    """
+
     # turn --subject/--variable/--exclude
     # arguments into lists of IDs. If
     # error is True, an error is raised on
@@ -690,14 +829,35 @@ def parseArgs(argv=None, namespace=None):
     # a list of integer IDs, but subject is
     # transformed into a tuple containing
     # ([ID], [exprStr])
-    if args.noisy    is     None: args.noisy    = 0
-    if args.quiet:                args.noisy    = 0
     if args.subject  is not None: args.subject  = replaceIDs(args.subject,
                                                              False)
     else:                         args.subject  = None, None
     if args.variable is not None: args.variable = replaceIDs(args.variable)[0]
     if args.exclude  is not None: args.exclude  = replaceIDs(args.exclude)[0]
 
+
+def _prepareCategorySelectors(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    the category selection options.
+
+    :arg args: ``argparse.Namespace`` object.
+    """
+
+    # categories can be specified
+    # either by name or by ID -
+    # convert the latter to integers.
+    if args.category is not None:
+        for i, c in enumerate(args.category):
+            try:               args.category[i] = int(c)
+            except ValueError: continue
+
+
+def _prepareColumnSelectors(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    the column selection options.
+
+    :arg args: ``argparse.Namespace`` object.
+    """
     # The column option accepts
     # column names, or a file
     # containing column names
@@ -712,6 +872,13 @@ def parseArgs(argv=None, namespace=None):
     if args.column is not None:
         args.column = list(it.chain(*[loadIfExists(c) for c in args.column]))
 
+
+def _prepareVisitSelectors(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    visit selection options.
+
+    :arg args: ``argparse.Namespace`` object.
+    """
     # visits are restricted using the
     # keepVisits cleaning function
     if args.visit is not None:
@@ -725,14 +892,13 @@ def parseArgs(argv=None, namespace=None):
         if args.global_clean is None: args.global_clean  =       visit
         else:                         args.global_clean += ',' + visit
 
-    # categories can be specified
-    # either by name or by ID -
-    # convert the latter to integers.
-    if args.category is not None:
-        for i, c in enumerate(args.category):
-            try:               args.category[i] = int(c)
-            except ValueError: continue
 
+def _prepareCleaningSelectors(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    cleaning function options.
+
+    :arg args: ``argparse.Namespace`` object.
+    """
     def numlist(s):
         return np.fromstring(s, sep=',', dtype=np.float)
 
@@ -765,88 +931,26 @@ def parseArgs(argv=None, namespace=None):
     if args.type_clean is not None:
         args.type_clean = {util.CTYPES[t] : e for t, e in args.type_clean}
 
-    # The variable_file, datacoding_file, type_file,
-    # processing_file, and category_file options
-    # can be specified relative to funpackdir
-    configdir = op.join(op.dirname(__file__), 'configs')
 
-    def fixPath(f):
+def _prepareIDsOnly(args):
+    """Sub-function of :func:`parseArgs`. Prepares arguments related to
+    the --ids_only option.
 
-        if f is None:
-            return f
-
-        options = [
-            f,
-            op.join(configdir, f),
-            op.join(configdir, f.replace('.', op.sep) + '.tsv')
-        ]
-
-        for o in options:
-            if op.exists(o):
-                return o
-
-        # if the fixed version does not
-        # exist, allow processing to
-        # continue - it will fail later on.
-        return f
-
-    if args.variable_file is not None:
-        args.variable_file = [fixPath(f) for f in args.variable_file]
-    if args.datacoding_file is not None:
-        args.datacoding_file = [fixPath(f) for f in args.datacoding_file]
-
-    args.type_file       = fixPath(args.type_file)
-    args.processing_file = fixPath(args.processing_file)
-    args.category_file   = fixPath(args.category_file)
-
-    return args, argv
-
-
-def loadConfigFile(cfgfile, namespace=None):
-    """Loads arguments from the given configuration file, returning an
-    ``argparse.Namespace`` object.
-
-    :arg cfgfile:   Path to configuration file
-    :arg namespace: Existing ``argparse.Namespace`` object to merge
-                    arguments into.
+    :arg args: ``argparse.Namespace`` object.
     """
+    if not args.ids_only:
+        return
 
-    argv = []
+    args.skip_insertna      = True
+    args.skip_childvalues   = True
+    args.skip_clean_funcs   = True
+    args.skip_recoding      = True
+    args.skip_processing    = True
 
-    # If the specified config file does
-    # not exist, assume it is a reference
-    # to a built-in config file
-    if not op.exists(cfgfile):
-        if not cfgfile.endswith('.cfg'):
-            cfgfile = cfgfile + '.cfg'
-        moddir   = op.dirname(op.abspath(__file__))
-        cfgfile = op.join(moddir, 'configs', cfgfile)
-
-    log.debug('Loading arguments from configuration file %s', cfgfile)
-
-    with open(cfgfile, 'rt') as f:
-        lines = [line.strip() for line in f.readlines()]
-        lines = [line         for line in lines if line != '']
-        lines = [line         for line in lines if not line.startswith('#')]
-
-    for line in lines:
-        words        = list(shlex.split(line))
-        name, values = words[0], words[1:]
-
-        argv.append('--{}'.format(name))
-        argv.extend(values)
-
-    argv      = sanitiseArgs(argv)
-    parser    = makeParser(exclude=('outfile', 'infile'))
-    namespace = parser.parse_args(argv, namespace)
-
-    # a config file may "include" other
-    # config files - we parse them recursively,
-    # and merge the arguments in load order.
-    cfgfiles = namespace.config_file
-    if cfgfiles is not None and len(cfgfiles) > 0:
-        for cf in cfgfiles:
-            namespace.config_file = None
-            namespace             = loadConfigFile(cf, namespace)
-
-    return namespace
+    # See main.main, and the do* functions
+    # that it calls - aux file export is
+    # ultimately toggled by these arguments
+    args.unknown_vars_file  = None
+    args.icd10_map_file     = None
+    args.description_file   = None
+    args.summary_file       = None
