@@ -363,12 +363,18 @@ def parseVariable(toks):
     return toks[1]
 
 
-# conditionals are constructed to produce
-# numpy arrays, so that is what these
-# operations expect as inputs
-def _not(op,       *args): return ~op(*args)               # noqa
-def _and(op1, op2, *args): return op1(*args) & op2(*args)  # noqa
-def _or( op1, op2, *args): return op1(*args) | op2(*args)  # noqa
+# These functions implement the logic for
+# evaluating unary and binary expressions.
+# They must be module-level functions, as
+# they must be pickle-able.
+#
+# Conditional expressions are constructed to
+# produce numpy arrays (see parseCondition
+# and its use of _asarray), so that is what
+# these unary/binary operations expect as
+# einputs.
+def _not(op, *args):
+    return ~op(*args)
 
 def _any(op, *args):
     result = op(*args)
@@ -380,6 +386,24 @@ def _all(op, *args):
     if len(result.shape) == 2: return result.all(axis=1)
     else:                      return result
 
+def _and(op1, op2, *args):
+    op1, op2 = _evalBinaryOperands(op1, op2, *args)
+    return op1 & op2
+
+def _or( op1, op2, *args):
+    op1, op2 = _evalBinaryOperands(op1, op2, *args)
+    return op1 | op2
+
+def _evalBinaryOperands(op1, op2, *args):
+    op1 = op1(*args)
+    op2 = op2(*args)
+
+    if op1.shape != op2.shape:
+        if len(op1.shape) == 2: op1 = op1.any(axis=1)
+        if len(op2.shape) == 2: op2 = op2.any(axis=1)
+
+    return op1, op2
+
 
 def parseUnary(toks):
     """Called by the parser created by :func:`makeParser`. Parses an expression of
@@ -414,6 +438,23 @@ def parseBinary(toks):
     dictionary, and ``expression1`` and ``expression2`` are conditional
     statements or logical expression.
 
+    Binary expressions expect that the shape of both operands is equal;
+    the number of rows is guaranteed to match (because ultimately the
+    operands are coming from the same ``pandas.DataFrame``. But the
+    number of columns may differ if, for example, one operand has been
+    calculated from a multi-valued variable, and another from a single-
+    valued variable.
+
+    The outcome of this situation can be explicitly controlled in the query
+    by use of the ``any`` and ``all`` operators, which can be used to
+    collapse the columns of a variable down to a single column.
+
+    But if this is not explicitly controlled, the default behaviour which
+    occurs when the operands of a binary operator have a different number of
+    columns is to collapse both operands down to a single column via the
+    ``any`` operator - in other words, combining values within each row witha
+    logical "or" operation.
+
     Returns a function which can be used to evaluate the expression.
     """
 
