@@ -11,6 +11,7 @@ the data importing stage of the ``funpack`` sequence
 
 import itertools             as it
 import functools             as ft
+import multiprocessing       as mp
 import multiprocessing.dummy as mpd
 import                          logging
 import                          warnings
@@ -77,8 +78,9 @@ def importData(datafiles,
                mergeAxis=None,
                mergeStrategy=None,
                indexVisits=False,
+               dropNaRows=False,
                loaders=None,
-               pool=None,
+               njobs=1,
                mgr=None,
                dryrun=False):
     """The data import stage.
@@ -92,70 +94,72 @@ def importData(datafiles,
 
       3. Creates and returns a :class:`DataTable`.
 
-    :arg datafiles:         Path to the data file(s)
+    :arg datafiles:     Path to the data file(s)
 
-    :arg vartable:          The data coding table
+    :arg vartable:      The data coding table
 
-    :arg proctable:         The processing table
+    :arg proctable:     The processing table
 
-    :arg cattable:          The category table
+    :arg cattable:      The category table
 
-    :arg variables:         List of variable IDs to import
+    :arg variables:     List of variable IDs to import
 
-    :arg colnames:          List of names/glob-style wildcard patterns
-                            specifying columns to import.
+    :arg colnames:      List of names/glob-style wildcard patterns
+                        specifying columns to import.
 
-    :arg categories:        List of category names to import
+    :arg categories:    List of category names to import
 
-    :arg subjects:          List of subjects to include
+    :arg subjects:      List of subjects to include
 
-    :arg subjectExprs:      List of subject inclusion expressions
+    :arg subjectExprs:  List of subject inclusion expressions
 
-    :arg exclude:           List of subjects to exclude
+    :arg exclude:       List of subjects to exclude
 
-    :arg encoding:          Character encoding(s) for data file(s). See
-                            :func:`loadData`.
+    :arg encoding:      Character encoding(s) for data file(s). See
+                        :func:`loadData`.
 
-    :arg trustTypes:        If ``True``, it is assumed that columns with a
-                            known data type do not contain any bad/unparseable
-                            values. This improves performance, but will cause
-                            an error if the assumption does not hold.
+    :arg trustTypes:    If ``True``, it is assumed that columns with a
+                        known data type do not contain any bad/unparseable
+                        values. This improves performance, but will cause
+                        an error if the assumption does not hold.
 
-    :arg indexes:           Dict of ``{filename : [index]}`` mappings,
-                            specifying the position of the column(s) to use as
-                            the index. Defaults to 0 (the first column).
+    :arg indexes:       Dict of ``{filename : [index]}`` mappings,
+                        specifying the position of the column(s) to use as
+                        the index. Defaults to 0 (the first column).
 
-    :arg mergeAxis:         Merging axis to use when loading multiple data
-                            files - see the :func:`mergeData` function.
+    :arg mergeAxis:     Merging axis to use when loading multiple data
+                        files - see the :func:`mergeData` function.
 
-    :arg mergeStrategy:     Merging strategy to use when loading multiple
-                            data files - see the :func:`mergeData` function.
+    :arg mergeStrategy: Merging strategy to use when loading multiple
+                        data files - see the :func:`mergeData` function.
 
-    :arg indexVisits:       Re-arrange the data so that rows are indexed by
-                            subject ID and visit, rather than visits being
-                            split into separate columns. Only applied to
-                            variables which are labelled with Instancing 2.
+    :arg indexVisits:   Re-arrange the data so that rows are indexed by
+                        subject ID and visit, rather than visits being
+                        split into separate columns. Only applied to
+                        variables which are labelled with Instancing 2.
 
-    :arg loaders:           Dict of ``{ file : loaderName }`` mappings
-                            containing custom sniffers/loaders to be used for
-                            specific files. See the :mod:`.custom` module.
+    :arg dropNaRows:    If ``True``, rows which do not contain data for any
+                        columns are not loaded.
 
-    :arg pool:              ``multiprocessing.Pool`` to use for parallelising
-                            tasks.
+    :arg loaders:       Dict of ``{ file : loaderName }`` mappings
+                        containing custom sniffers/loaders to be used for
+                        specific files. See the :mod:`.custom` module.
 
-    :arg mgr:               ``multiprocessing.Manager`` to use for sharing
-                            state.
+    :arg njobs:         Number of processes to use for parallelising tasks.
 
-    :arg dryrun:            If ``True`` the data is not loaded.
+    :arg mgr:           :class:`multiprocessing.Manager` object for
+                        parallelisation
 
-    :returns:               A tuple containing:
+    :arg dryrun:        If ``True`` the data is not loaded.
 
-                             - A :class:`DataTable`, which contains references
-                               to the data, and the variable and procesing
-                               tables.
+    :returns:           A tuple containing:
 
-                             - A list of :class:`.Column` objects that were not
-                               loaded from each input file.
+                         - A :class:`DataTable`, which contains references
+                           to the data, and the variable and procesing
+                           tables.
+
+                         - A list of :class:`.Column` objects that were not
+                           loaded from each input file.
     """
 
     variables = filter.restrictVariables(cattable, variables, categories)
@@ -170,22 +174,28 @@ def importData(datafiles,
 
     # Load those columns, merging
     # multiple input files.
-    data, cols = loadFiles(datafiles,
-                           vartable,
-                           cols,
-                           subjects=subjects,
-                           subjectExprs=subjectExprs,
-                           exclude=exclude,
-                           encoding=encoding,
-                           indexes=indexes,
-                           mergeAxis=mergeAxis,
-                           mergeStrategy=mergeStrategy,
-                           indexVisits=indexVisits,
-                           loaders=loaders,
-                           trustTypes=trustTypes,
-                           pool=pool,
-                           mgr=mgr,
-                           dryrun=dryrun)
+    if njobs > 1: Pool = mp.Pool
+    else:         Pool = mpd.Pool
+
+    with Pool(njobs) as pool:
+        data, cols = loadFiles(datafiles,
+                               vartable,
+                               cols,
+                               subjects=subjects,
+                               subjectExprs=subjectExprs,
+                               exclude=exclude,
+                               encoding=encoding,
+                               indexes=indexes,
+                               mergeAxis=mergeAxis,
+                               mergeStrategy=mergeStrategy,
+                               indexVisits=indexVisits,
+                               dropNaRows=dropNaRows,
+                               loaders=loaders,
+                               trustTypes=trustTypes,
+                               pool=pool,
+                               dryrun=dryrun)
+        pool.close()
+        pool.join()
 
     # Re-order the columns according to
     # specified variables, if provided
@@ -218,7 +228,7 @@ def importData(datafiles,
             data = data[[c.name for c in cols[1:]]]
 
     dtable = datatable.DataTable(
-        data, cols, vartable, proctable, cattable, pool, mgr)
+        data, cols, vartable, proctable, cattable, njobs, mgr)
 
     return dtable, drop
 
@@ -236,9 +246,9 @@ def loadFiles(datafiles,
               mergeAxis=None,
               mergeStrategy=None,
               indexVisits=False,
+              dropNaRows=False,
               loaders=None,
               pool=None,
-              mgr=None,
               dryrun=False):
     """Load data from ``datafiles``, using :func:`.mergeDataFrames` if multiple
     files are provided.
@@ -283,6 +293,9 @@ def loadFiles(datafiles,
                         separate columns. Only applied to variables which are
                         labelled with Instancing 2.
 
+    :arg dropNaRows:    If ``True``, rows which do not contain data for any
+                        columns are not loaded.
+
     :arg loaders:       Dict of ``{ file : loaderName }`` mappings containing
                         custom loaders/sniffers to be used for specific files.
                         See the :mod:`.custom` module.
@@ -290,8 +303,6 @@ def loadFiles(datafiles,
     :arg pool:          ``multiprocessing.Pool`` object for running tasks in
                         parallel.
 
-    :arg mgr:           ``multiprocessing.Manager`` to use for sharing state.
-
     :arg dryrun:        If ``True``, the data is not loaded.
 
     :returns:           A tuple containing:
@@ -347,10 +358,10 @@ def loadFiles(datafiles,
                                     subjectExprs=subjectExprs,
                                     exclude=exclude,
                                     indexVisits=indexVisits,
+                                    dropNaRows=dropNaRows,
                                     encoding=fencoding,
                                     trustTypes=trustTypes,
-                                    pool=pool,
-                                    mgr=mgr)
+                                    pool=pool)
 
         data      .append(fdata)
         loadedcols.append(fcols)
@@ -396,10 +407,10 @@ def loadFile(fname,
              subjectExprs=None,
              exclude=None,
              indexVisits=False,
+             dropNaRows=False,
              encoding=None,
              trustTypes=False,
-             pool=None,
-             mgr=None):
+             pool=None):
     """Loads data from the specified file. The file is loaded in chunks of
     ``nrows`` rows, using the :func:`loadChunk` file.
 
@@ -436,6 +447,9 @@ def loadFile(fname,
                        separate columns. Only applied to variables which are
                        labelled with Instancing 2.
 
+    :arg dropNaRows:   If ``True``, rows which do not contain data for any
+                       columns are not loaded.
+
     :arg encoding:     Character encoding (or sequence of encodings, one
                        for each data file). Defaults to ``latin1``.
 
@@ -445,8 +459,6 @@ def loadFile(fname,
     :arg pool:         ``multiprocessing.Pool`` object for running tasks in
                        parallel.
 
-    :arg mgr:          ``multiprocessing.Manager`` to use for sharing state.
-
     :returns:          A tuple containing:
                         - A ``pandas.DataFrame`` containing the data.
                         - A list of :class:`.Column` objects representing the
@@ -522,6 +534,7 @@ def loadFile(fname,
             'subjectExprs' : subjectExprs,
             'exclude'      : exclude,
             'indexByVisit' : indexVisits,
+            'dropNaRows'   : dropNaRows,
             'encoding'     : encoding,
             'trustTypes'   : trustTypes,
             'dtypes'       : dtypes,
@@ -582,6 +595,7 @@ def loadChunk(i,
               subjectExprs,
               exclude,
               indexByVisit,
+              dropNaRows,
               encoding,
               trustTypes,
               dtypes,
@@ -624,6 +638,9 @@ def loadChunk(i,
                        to re-arrange the loaded data so it is indexed by
                        visit number, in addition to row ID.
 
+    :arg dropNaRows:   If ``True``, rows which do not contain data for any
+                       columns are not loaded.
+
     :arg encoding:     Character encoding (or sequence of encodings, one
                        for each data file). Defaults to ``latin1``.
 
@@ -667,6 +684,10 @@ def loadChunk(i,
                          **dlargs)
         gotrows = len(df)
 
+        # drop NA rows if requested
+        if dropNaRows:
+            df.dropna(how='all', inplace=True)
+
         # If a subject/expression/exclude list
         # is provided, filter the rows accordingly
         df = filter.filterSubjects(df, toload, subjects, subjectExprs, exclude)
