@@ -8,7 +8,6 @@
 
 
 import multiprocessing     as mp
-import functools           as ft
 import os.path             as op
 import                        sys
 import                        logging
@@ -18,6 +17,7 @@ import                        calendar
 
 import pandas              as pd
 import pandas.api.types    as pdtypes
+import                        threadpoolctl
 
 import funpack
 import funpack.util        as util
@@ -26,11 +26,12 @@ import funpack.config      as config
 import funpack.custom      as custom
 import funpack.dryrun      as dryrun
 import funpack.cleaning    as cleaning
-import funpack.importing   as importing
+import funpack.datatable   as datatable
 import funpack.exporting   as exporting
 import funpack.hierarchy   as hierarchy
-import funpack.processing  as processing
+import funpack.importing   as importing
 import funpack.loadtables  as loadtables
+import funpack.processing  as processing
 
 
 log = logging.getLogger(__name__)
@@ -50,6 +51,10 @@ def main(argv=None):
     args, argv = config.parseArgsWithConfigFile(argv)
     date = datetime.date.today()
 
+    # Set the number of threads
+    # that numpy should use
+    threadpoolctl.threadpool_limits(args.num_jobs)
+
     # Now that args are passed,
     # we can set up logging properly.
     configLogging(args)
@@ -111,46 +116,31 @@ def main(argv=None):
         # are created, so its state is
         # shared by all processes.
         icd10.initialise(mgr)
-
-        pool = mp.Pool(args.num_jobs)
-
     else:
-        pool = None
-        mgr  = None
+        mgr = None
 
-    try:
-        with util.timed(None, log, fmt='Total time: %s (%+iMB)'):
+    with util.timed(None, log, fmt='Total time: %s (%+iMB)'):
 
-            dtable, unknowns, uncategorised, drop = doImport(args, pool, mgr)
+        dtable, unknowns, uncategorised, drop = doImport(args, mgr)
 
-            if args.dry_run:
-                dryrun.doDryRun(dtable, unknowns, uncategorised, drop, args)
-            else:
-                doCleanAndProcess(  dtable, args)
-                doFormat(           dtable, args)
-                doUnknownsExport(   dtable, args, unknowns, uncategorised)
-                doExport(           dtable, args)
-                doICD10Export(              args)
-                doDescriptionExport(dtable, args)
-                doSummaryExport(    dtable, args)
-
-    finally:
-        # shutdown the pool gracefully
-        if pool is not None:
-            pool.close()
-            pool.join()
-            pool = None
+        if args.dry_run:
+            dryrun.doDryRun(dtable, unknowns, uncategorised, drop, args)
+        else:
+            doCleanAndProcess(  dtable, args)
+            doExport(           dtable, args)
+            doUnknownsExport(   dtable, args, unknowns, uncategorised)
+            doICD10Export(              args)
+            doDescriptionExport(dtable, args)
+            doSummaryExport(    dtable, args)
 
     return 0
 
 
-def doImport(args, pool, mgr):
+def doImport(args, mgr):
     """Data import stage.
 
     :arg args: :class:`argparse.Namespace` object containing command line
                arguments
-    :arg pool: :class:`multiprocessing.Pool` object for parallelisation (may
-               be ``None``)
     :arg mgr:  :class:`multiprocessing.Manager` object for parallelisation (may
                be ``None``)
 
@@ -212,8 +202,9 @@ def doImport(args, pool, mgr):
             mergeAxis=args.merge_axis,
             mergeStrategy=args.merge_strategy,
             indexVisits=args.index_visits,
+            dropNaRows=args.drop_na_rows,
             loaders=args.loader,
-            pool=pool,
+            njobs=args.num_jobs,
             mgr=mgr,
             dryrun=args.dry_run)
 
@@ -270,59 +261,75 @@ def doCleanAndProcess(dtable, args):
         processing.processData(dtable)
 
 
-def doFormat(dtable, args):
-    """Data formatting stage. This is called just prior to :func:`doExport`.
+def splitDataTable(dtable, args):
+    """Splits the .:class:`DataTable` into separate numeric/non-numeric tables.
 
-    It applies any variable-specific formatting that has been specified, and
-    also formats date/time columns.
+    Called by :func:`doExport`.  If the ``--suppress_non_numerics`` and/or
+    ``--write_non_numerics`` options are active, non-numeric columns need to
+    be separated from numeric columns, and possibly saved to a separate output
+    file.
 
     :arg dtable: :class:`.DataTable` containing the data
     :arg args:   :class:`argparse.Namespace` object containing command line
                  arguments
+    :returns:    A list of ``(DataTable, filename)`` tuples, containing the
+                 :class:`.DataTable` instances and corresponding output
+                 file names.
     """
 
-    vartable   = dtable.vartable
-    dateFormat = args.date_format
-    timeFormat = args.time_format
-    formatters = args.var_format
+    # We're outputting one main output file
+    # with all columns - no need to split
+    if not (args.suppress_non_numerics or args.write_non_numerics):
+        return [(dtable, args.outfile)]
 
-    # We format columns in parallel - build
-    # a list of arguments to pass to the
-    # exporting.formatColumn function.
-    fmtargs = []
-    fmtfunc = ft.partial(exporting.formatColumn,
-                         dateFormat=dateFormat,
-                         timeFormat=timeFormat,
-                         formatters=formatters)
+    # we need to separate out numeric from
+    # non-numeric columns, and potentially
+    # create two separate datatables
+    dtables = []
+    ncols   = []
+    nncols  = []
 
-    # We format all date/time columns, and
-    # any for which a custom formatter or
-    # a fill value has been provided.
+    # we run a single value from each
+    # column through formatting in
+    # order to determine whether each
+    # column is numeric or non-numeric
     for col in dtable.dataColumns:
 
-        vid    = col.vid
-        name   = col.name
-        series = dtable[:, name]
+        idx = dtable[:, col.name].first_valid_index()
 
-        if vid in vartable.index: vtype = vartable['Type'][vid]
-        else:                     vtype = None
+        # column is all nan
+        if idx is None:
+            continue
 
-        if any((vid   in formatters,
-                name  in formatters,
-                vtype in (util.CTYPES.date, util.CTYPES.time),
-                pdtypes.is_datetime64_any_dtype(series),
-                col.fillval is not None)):
-            fmtargs.append((col, dtable.subtable([col])))
+        # format a subtable containing
+        # just the column and the first
+        # non-na value
+        series = exporting.formatColumn(
+            col,
+            dtable.subtable([col], [idx]),
+            dateFormat=args.date_format,
+            timeFormat=args.time_format,
+            formatters=args.var_format)[:, col.name]
 
-    # Format each column in parallel - each
-    # exporting.formatColumn call returns a
-    # subtable that can then be merged back
-    # in to the main table.
-    with dtable.pool() as pool:
-        results = pool.starmap(fmtfunc, fmtargs)
+        # separate accordingly
+        if pdtypes.is_numeric_dtype(series): ncols .append(col)
+        else:                                nncols.append(col)
 
-    for subtable in results:
-        dtable.merge(subtable)
+    # if suppress, only numeric columns
+    # are saved to main output file
+    if args.suppress_non_numerics:
+        log.debug('Separating out %u numeric columns for export', len(ncols))
+        dtables.append((dtable.subtable(ncols), args.outfile))
+    else:
+        dtables.append((dtable, args.outfile))
+
+    # if write, non-numeric columns
+    # are saved to an auxillary file
+    if args.write_non_numerics:
+        log.debug('Separating out %u non-numeric columns for export', len(nncols))
+        dtables.append((dtable.subtable(nncols), args.non_numerics_file))
+
+    return dtables
 
 
 def doExport(dtable, args):
@@ -333,40 +340,22 @@ def doExport(dtable, args):
                  arguments
     """
 
+    # if ids_only is specified, all we
+    # need to do is output the index
+    if args.ids_only:
+        with open(args.outfile, 'wt') as f:
+            for i in dtable.index:
+                f.write(f'{i}\n')
+        return
+
+    # Otherwise we are exporting the full
+    # data set, and things become more
+    # complicated...
+
     # List of data tables and file names
     # for export (we may split the dtable
-    # up into two)
-    dtables = []
-
-    # Suppress/separate out non-
-    # numeric columns if requested
-    if args.suppress_non_numerics or args.write_non_numerics:
-
-        # identify numeric/non-
-        # numeric columns
-        ncols  = []
-        nncols = []
-        for col in dtable.dataColumns:
-            series = dtable[:, col.name]
-            if pdtypes.is_numeric_dtype(series): ncols .append(col)
-            else:                                nncols.append(col)
-
-        # if suppress, only numeric
-        # columns are saved to main
-        # output file
-        if args.suppress_non_numerics:
-            dtables.append((dtable.subtable(ncols), args.outfile))
-        else:
-            dtables.append((dtable, args.outfile))
-
-        # if write, non-numeric columns
-        # are saved to an auxillary file
-        if args.write_non_numerics:
-            dtables.append((dtable.subtable(nncols), args.non_numerics_file))
-
-    # only one output file
-    else:
-        dtables.append((dtable, args.outfile))
+    # up into two - numeric/non-numeric)
+    dtables = splitDataTable(dtable, args)
 
     # If not parallelising, we export the
     # entire file in one go. Because what's
@@ -377,6 +366,12 @@ def doExport(dtable, args):
 
     with util.timed('Data export', log):
         for dtable, outfile in dtables:
+
+            # set the DataTable singleton for
+            # shared mem to child processes
+            # during export.
+            datatable.DataTable.setInstance(dtable)
+
             exporting.exportData(
                 dtable,
                 outfile,
@@ -384,6 +379,10 @@ def doExport(dtable, args):
                 # General export options
                 fileFormat=args.format,
                 numRows=args.num_rows,
+                dropNaRows=args.drop_na_rows,
+                dateFormat=args.date_format,
+                timeFormat=args.time_format,
+                formatters=args.var_format,
 
                 # TSV options
                 sep=args.tsv_sep,
