@@ -59,7 +59,7 @@ the following:
 
         independent,1,2,3 processName
 
-  - ``'all'``: The process is applied to all vids.::
+  - ``'all'``: The process is applied to all vids::
 
         all processName
 
@@ -303,8 +303,9 @@ def runProcess(proc, dtable, vids, workDir, parallel, bcastIdxs):
             # to columns, and column flags/
             # metadata. Added/removed columns
             # are handled below.
-            for subtable in subtables:
-                dtable.merge(subtable)
+            log.debug('Processing for %u vid groups complete - merging '
+                      'results into main data table.', len(subtables))
+            dtable.merge(subtables)
 
     remove    = []
     add       = []
@@ -324,7 +325,7 @@ def runProcess(proc, dtable, vids, workDir, parallel, bcastIdxs):
         savedSeries = list(glob.glob(
             op.join(workDir, '**', '*.pkl'), recursive=True))
         if len(savedSeries) > 0:
-            log.debug('[%s]: Reading %u new column from %s %s ...',
+            log.debug('[%s]: Reading %u new columns from %s %s ...',
                       proc.name, len(savedSeries), workDir, vids[:5])
 
     for i in range(len(add)):
