@@ -61,7 +61,6 @@ The following processing functions are defined:
 """
 
 
-import multiprocessing as mp
 import functools       as ft
 import itertools       as it
 import                    logging
@@ -71,25 +70,29 @@ import numpy            as np
 import pandas           as pd
 import pandas.api.types as pdtypes
 
+from typing import List, Optional, Any
+
 from . import processing_functions_core as core
 from . import                              util
 from . import                              custom
-
+from . import                              datatable
 
 log = logging.getLogger(__name__)
 
 
 @custom.processor()
-def removeIfSparse(dtable,
-                   vids,
-                   minpres=None,
-                   minstd=None,
-                   mincat=None,
-                   maxcat=None,
-                   abspres=True,
-                   abscat=True,
-                   naval=None,
-                   ignoreType=False):
+def removeIfSparse(
+        dtable     : datatable.DataTable,
+        vids       : List[int],
+        minpres    : Optional[float] = None,
+        minstd     : Optional[float] = None,
+        mincat     : Optional[float] = None,
+        maxcat     : Optional[float] = None,
+        abspres    : bool            = True,
+        abscat     : bool            = True,
+        naval      : Optional[Any]   = None,
+        ignoreType : bool            = False
+) -> List[datatable.Column]:
     """removeIfSparse([minpres], [minstd], [mincat], [maxcat], [abspres], [abscat], [naval])
     Removes columns deemed to be sparse.
 
@@ -102,31 +105,42 @@ def removeIfSparse(dtable,
     See the :func:`.isSparse` function for details on the other arguments.
     """  # noqa
 
-    remove = []
+    cols   = []
+    series = []
+    vtypes = []
 
     for vid in vids:
 
         if ignoreType: vtype = None
         else:          vtype = dtable.vartable.loc[vid, 'Type']
 
-        for col in dtable.columns(vid):
+        vcols = dtable.columns(vid)
 
-            log.debug('Checking column %s for sparsity', col.name)
+        cols  .extend(vcols)
+        series.extend([dtable[:, c.name] for c in vcols])
+        vtypes.extend([vtype] * len(vcols))
 
-            isSparse, test, val = core.isSparse(dtable[:, col.name],
-                                                vtype,
-                                                minpres=minpres,
-                                                minstd=minstd,
-                                                mincat=mincat,
-                                                maxcat=maxcat,
-                                                abspres=abspres,
-                                                abscat=abscat,
-                                                naval=naval)
+    log.debug('Checking %u columns for sparsity %s ...', len(series), vids[:5])
 
-            if isSparse:
-                log.debug('Dropping sparse column %s (%s: %f)',
-                          col.name, test, val)
-                remove.append(col)
+    func = ft.partial(core.isSparse,
+                      minpres=minpres,
+                      minstd=minstd,
+                      mincat=mincat,
+                      maxcat=maxcat,
+                      abspres=abspres,
+                      abscat=abscat,
+                      naval=naval)
+
+    with dtable.pool() as pool:
+        results = pool.starmap(func, zip(series, vtypes))
+
+    remove = []
+
+    for col, (isSparse, reason, value) in zip(cols, results):
+        if isSparse:
+            log.debug('Dropping sparse column %s (%s: %f)',
+                      col.name, reason, value)
+            remove.append(col)
 
     if len(remove) > 0:
         log.debug('Dropping %u sparse columns: %s ...',
@@ -136,91 +150,30 @@ def removeIfSparse(dtable,
 
 
 @custom.processor()
-def removeIfRedundant(dtable, vids, corrthres, nathres=None):
-    """removeIfRedundant(corrthres, [nathres])
+def removeIfRedundant(dtable, vids, corrthres, nathres=None, pairwise=False):
+    """removeIfRedundant(corrthres, [nathres], [pairwise])
     Removes columns deemed to be redundant.
 
     Removes columns from the variables in ``vids`` if they are redundant.
 
     Redundancy is determined by calculating the correlation between pairs
     of columns - see the :func:`.isRedundant` function.
+
+    If ``pairwise`` is ``True``, an alternative implementation is used which
+    may be faster on data sets with high missingness correlation.
     """
 
     # Ignore non-numeric columns
     cols     = list(it.chain(*[dtable.columns(v) for v in vids]))
     cols     = [c for c in cols if pdtypes.is_numeric_dtype(dtable[:, c.name])]
     colnames = [c.name for c in cols]
+    data     = dtable[:, colnames]
 
-    # evaluate all pairs at once
-    if not dtable.parallel:
-        colpairs = list(it.combinations(colnames, 2))
-        log.debug('Checking %u columns for redundancy', len(cols))
-        redundant = core.redundantColumns(
-            dtable[:, :], colpairs, corrthres=corrthres, nathres=nathres)
-
-    # evaluate in parallel
+    if pairwise:
+        redundant = _pairwiseRemoveIfRedundant(
+            dtable, data, corrthres, nathres)
     else:
-        # We need to compare every pair of
-        # columns. We parallelise this by
-        # splitting the columns into chunks,
-        # and then passing pairs of chunks
-        # to the redundantColumns function.
-        #
-        # We limit the size of each chunk to
-        # ~750MB - we have to transfer two
-        # chunks to each worker process, and
-        # versions of python <= 3.7 can only
-        # transfer up to 2GB between
-        # processes. The limit value is
-        # attached as an attribute of this
-        # function so it can be manipulated
-        # by unit tests.
-        chunks    = [[]]
-        chunksize = 0
-        for col in colnames:
-            colsize = dtable[:, col].memory_usage()
-            if (chunksize + colsize) > removeIfRedundant.CHUNK_SIZE_LIMIT:
-                chunks.append([col])
-                chunksize = colsize
-            else:
-                chunks[-1].append(col)
-                chunksize += colsize
-
-        # we build a list of column
-        # pairs, and views into the
-        # dataframe, for each chunk
-        chunkpairs = []
-        chunkdata  = []
-
-        # while doing so, we keep track
-        # of column pairs that have been
-        # assigned to a chunk, so we don't
-        # evaluate any pair more than once
-        assignedcolpairs = set()
-
-        # for every chunk, and
-        # every pair of chunks
-        for chunk1, chunk2 in it.combinations_with_replacement(chunks, 2):
-            chunkcols   = util.dedup(chunk1 + chunk2)
-            ichunkpairs = []
-            for colpair in it.combinations(chunkcols, 2):
-                if colpair not in assignedcolpairs:
-                    ichunkpairs     .append(colpair)
-                    assignedcolpairs.add(   colpair)
-            if len(ichunkpairs) > 0:
-                ichunkcols = util.dedup(it.chain(*ichunkpairs))
-                ichunkdata = dtable[:, ichunkcols]
-                chunkpairs.append(ichunkpairs)
-                chunkdata .append(ichunkdata)
-
-        log.debug('Checking %u columns for redundancy (%u tasks)',
-                  len(cols), len(chunkdata))
-
-        func = ft.partial(core.redundantColumns,
-                          corrthres=corrthres, nathres=nathres)
-        with dtable.pool() as pool:
-            redundant = pool.starmap(func, zip(chunkdata, chunkpairs))
-            redundant = list(it.chain(*redundant))
+        redundant = _removeIfRedundant(dtable, data, corrthres, nathres)
 
     redundant = util.dedup(sorted(redundant))
 
@@ -228,14 +181,109 @@ def removeIfRedundant(dtable, vids, corrthres, nathres=None):
         log.debug('Dropping %u redundant columns: %s ...',
                   len(redundant), redundant[:5])
 
-    cols = collections.OrderedDict(zip(colnames, cols))
     return [cols[r] for r in redundant]
 
 
-# maximum chunk size (in bytes)
-# that the removeIfRedundant function
-# will pass to worker processes
-removeIfRedundant.CHUNK_SIZE_LIMIT = 750000000
+def _removeIfRedundant(dtable, data, corrthres, nathres=None):
+    """Default fast implementation of redundancy check. Used when the
+    ``pairwise`` option to :func:`removeIfRedundant` is ``False``.
+
+    :arg dtable:    The :class:`.DataTable` containing all data
+    :arg data:      ``pandas.DataFrame`` containing the data to check
+    :arg corrthres: Correlation threshold - see :func:`.redundantColumns`.
+    :arg nathres:   Missingness correlation threshold - see
+                    :func:`.redundantColumns`.
+    :returns:       A sequence of indices denoting the redundant columns.
+    """
+
+    return core.matrixRedundantColumns(data, corrthres, nathres)
+
+
+def _pairwiseRemoveIfRedundant(dtable, data, corrthres, nathres=None):
+    """Alternative implementation of redundancy check. Used when the
+    ``pairwise`` option to :func:`removeIfRedundant` is ``True``.
+
+    :arg dtable:    The :class:`.DataTable` containing all data
+    :arg data:      ``pandas.DataFrame`` containing the data to check
+    :arg corrthres: Correlation threshold - see :func:`.redundantColumns`.
+    :arg nathres:   Missingness correlation threshold - see
+                    :func:`.redundantColumns`.
+    :returns:       A sequence of indices denoting the redundant columns.
+    """
+
+    ncols = len(data.columns)
+
+    # If we are correlating missingness,
+    # we use the naCorrelation function
+    # to identify all of the column pairs
+    # which are na-correlated - the pairs
+    # which fail this test do not need to
+    # be subjected to the correlation test
+    # (and therefore pass the redundancy
+    # check)
+    if nathres is not None:
+        nacorr = core.naCorrelation(pd.isna(data), nathres)
+
+        # As the matrix is symmetric, we can
+        # drop column pairs where x >= y.
+        nacorr   = np.triu(nacorr, k=1)
+        colpairs = np.where(nacorr)
+        colpairs = np.vstack(colpairs).T
+
+    # Otherwise we generate an array
+    # containing indices of all column
+    # pairs.
+    else:
+        xs, ys   = np.triu_indices(ncols, k=1)
+        colpairs = np.vstack((xs, ys)).T
+
+    # we need at least
+    # one pair of columns
+    if len(colpairs) == 0:
+        return []
+
+    # evaluate all pairs at once
+    if not dtable.parallel:
+        log.debug('Checking %u columns for redundancy', ncols)
+        redundant = core.pairwiseRedundantColumns(
+            data, colpairs, corrthres=corrthres)
+
+    # evaluate in parallel
+    else:
+        # Split the column pairs
+        # into njobs chunks, and
+        # run in parallel
+        chunksize  = int(np.ceil(len(colpairs) / dtable.njobs))
+        pairchunks = [colpairs[i:i + chunksize]
+                      for i in range(0, len(colpairs), chunksize)]
+
+        log.debug('Checking %u columns for redundancy (%u tasks)',
+                  ncols, len(pairchunks))
+
+        with dtable.pool() as pool:
+            results = []
+            for i, chunk in enumerate(pairchunks):
+
+                # We can pass the full dataframe
+                # to each task, as it should be
+                # read-accessible via shared memory.
+                token  = 'task {} / {}'.format(i + 1, len(pairchunks))
+                result = pool.apply_async(
+                    core.pairwiseRedundantColumns,
+                    kwds=dict(data=data,
+                              colpairs=chunk,
+                              corrthres=corrthres,
+                              token=token))
+                results.append(result)
+
+            # wait for the tasks to complete,
+            # and gather the results (indices
+            # of redundant columns) into a list
+            redundant = []
+            for result in results:
+                redundant.extend(result.get())
+
+    return redundant
 
 
 @custom.processor(auxvids=['take'])
@@ -256,6 +304,10 @@ def binariseCategorical(dtable,
     one new column for each value, containing ``1`` for subjects
     with that value, and ``0`` otherwise.
 
+    The :func:`.processing_functions_core.binariseCategorical` function is called
+    for each variable in ``vids`` - it internally parallelises generation of the
+    new columns using multiprocessing.
+
     :arg dtable:          The :class:`.DataTable`
 
     :arg vids:            Sequence of variable IDs to (independently) apply the
@@ -361,13 +413,18 @@ def binariseCategorical(dtable,
 
         for cols, takecols in zip(colgrps, takegrps):
 
+            log.debug('Calling binariseCategorical (vid: %i, '
+                      '%u columns)', vid, len(cols))
+
             if takecols is None: tkdata = None
             else:                tkdata = dtable[:, [c.name for c in takecols]]
 
             data              = dtable[:, [c.name for c in cols]]
             binarised, values = core.binariseCategorical(data,
                                                          minpres=minpres,
-                                                         take=tkdata)
+                                                         take=tkdata,
+                                                         token=vid,
+                                                         njobs=dtable.njobs)
 
             if replace:                                remove.extend(cols)
             if replaceTake and (takecols is not None): remove.extend(takecols)
