@@ -100,7 +100,7 @@ def test_keepVisits_not_instancing_2():
 
 def test_fillVisits():
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         mgr = mp.Manager()
 
@@ -146,7 +146,7 @@ def test_fillVisits():
                                          vartable,
                                          proctable,
                                          cattable,
-                                         pool=pool,
+                                         njobs=mp.cpu_count(),
                                          mgr=mgr)
 
             cfns.fillVisits(dt, 1, method)
@@ -167,7 +167,7 @@ def test_fillVisits():
 
 
 def test_fillMissing():
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         mgr = mp.Manager()
 
@@ -195,7 +195,7 @@ def test_fillMissing():
                                      vartable,
                                      proctable,
                                      cattable,
-                                     pool=pool,
+                                     njobs=mp.cpu_count(),
                                      mgr=mgr)
 
         cfns.fillMissing(dt, 1, 999)
@@ -252,7 +252,7 @@ def test_makeNa():
 
     cols = ['eid', '1-0.0', '2-0.0']
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         mgr = mp.Manager()
         with open('data.txt', 'wt') as f:
@@ -264,7 +264,7 @@ def test_makeNa():
                                      vartable,
                                      proctable,
                                      cattable,
-                                     pool=pool,
+                                     njobs=mp.cpu_count(),
                                      mgr=mgr)
 
         cfns.makeNa(dt, 1, '>= 25')
@@ -359,7 +359,7 @@ def test_applyNAInsertion():
 
     cols = ['eid', '1-0.0', '2-0.0']
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         mgr = mp.Manager()
         with open('data.txt', 'wt') as f:
@@ -370,7 +370,7 @@ def test_applyNAInsertion():
                                      vartable,
                                      proctable,
                                      cattable,
-                                     pool=pool,
+                                     njobs=mp.cpu_count(),
                                      mgr=mgr)
 
         cleaning.applyNAInsertion(dt)
@@ -436,7 +436,7 @@ def test_applyChildValues():
     exp[np.isnan(exp[:, 3]) & (exp[:, 4] >= 6), 3] = 300
     exp[np.isnan(exp[:, 4]) & (exp[:, 5] == 2), 4] = 400
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         mgr = mp.Manager()
         with open('data.txt', 'wt') as f:
@@ -454,7 +454,7 @@ def test_applyChildValues():
                                      vartable,
                                      proctable,
                                      cattable,
-                                     pool=pool,
+                                     njobs=mp.cpu_count(),
                                      mgr=mgr)
 
         cleaning.applyChildValues(dt)
@@ -488,7 +488,7 @@ def test_applyCleaningFunctions():
             dtable[:, c.name] = dtable[:, c.name] * factor
 
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         mgr = mp.Manager()
         with open('data.txt', 'wt') as f:
@@ -507,7 +507,7 @@ def test_applyCleaningFunctions():
                                      vartable,
                                      proctable,
                                      cattable,
-                                     pool=pool,
+                                     njobs=mp.cpu_count(),
                                      mgr=mgr)
 
         cleaning.applyCleaningFunctions(dt)
@@ -527,7 +527,7 @@ def test_cleanData():
 
     cols = ['eid', '1-0.0', '2-0.0']
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         mgr = mp.Manager()
         custom.registerBuiltIns()
@@ -539,7 +539,7 @@ def test_cleanData():
                                      vartable,
                                      proctable,
                                      cattable,
-                                     pool=pool,
+                                     njobs=mp.cpu_count(),
                                      mgr=mgr)
 
         # not crashing -> pass
@@ -567,7 +567,7 @@ def test_parseSpirometryData():
     exp = [[np.array(row) for row in col] for col in exp]
     exp = [pd.Series(col) for col in exp]
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
         with open('data.txt', 'wt') as f:
             f.write(data)
 
@@ -578,7 +578,7 @@ def test_parseSpirometryData():
                                      vartable,
                                      proctable,
                                      cattable,
-                                     pool=pool,
+                                     njobs=mp.cpu_count(),
                                      mgr=mgr)
 
         cfns.parseSpirometryData(dt, 1)
@@ -630,7 +630,7 @@ def test_flattenHierarchical():
         (4, [10, 20, 30, 40, 50, 60, 70, 80, 90]),
     ]
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         with open('data.txt', 'wt') as f:
             f.write(data)
@@ -647,7 +647,7 @@ def test_flattenHierarchical():
                                          vartable,
                                          proctable,
                                          cattable,
-                                         pool=pool,
+                                         njobs=mp.cpu_count(),
                                          mgr=mgr)
 
             with mock.patch('funpack.hierarchy.getHierarchyFilePath',
@@ -661,4 +661,3 @@ def test_flattenHierarchical():
 
         dt = None
         mgr = None
-        pool = None
