@@ -1503,3 +1503,52 @@ def test_main_varformats():
         assert np.all(got['1-0.0'] == expdates)
         assert np.all(got['2-0.0'] == exptimes)
         assert np.all(got['3-0.0'] == expfloats)
+
+
+@patch_logging
+def test_dropNaRows():
+    data = tw.dedent("""
+    eid,1-0.0,2-0.0
+    1,10,100
+    2,,
+    3,30,
+    4,40,400
+    5,,500
+    6,60,600
+    7,,
+    8,80,800
+    9,,
+    10,100,1000
+    """).strip()
+
+    with tempdir():
+
+        with open('in.csv', 'wt') as f:
+            f.write(data)
+
+        main.main('out1.csv in.csv'.split())
+        main.main('out2.csv in.csv -dn'.split())
+
+        got1 = pd.read_csv('out1.csv', sep=',', index_col=0)
+        got2 = pd.read_csv('out2.csv', sep=',', index_col=0)
+        exp1 = pd.read_csv('in.csv',   sep=',', index_col=0)
+        exp2 = exp1.dropna(how='all')
+
+        assert_frame_equal(got1, exp1)
+        assert_frame_equal(got2, exp2)
+
+
+@patch_logging
+def test_ids_only():
+    with tempdir():
+        gen_test_data(10, 10, 'data.tsv')
+
+        # aux file export should be disabled
+        main.main('-io -wu -wim -wde -ws out.txt data.tsv'.split())
+
+        assert sorted(os.listdir()) == ['data.tsv', 'out.txt']
+
+        exp = '\n'.join([f'{i}' for i in range(1, 11)])
+        with open('out.txt') as f:
+            got = f.read()
+        assert got.strip() == exp
