@@ -50,7 +50,7 @@ def test_processData():
     procs  = [processing.parseProcesses(p, 'processor') for p in procs]
     procs  = [{p[0].name : p[0]} for p in procs]
 
-    with tempdir(), mp.Pool(mp.cpu_count()) as pool:
+    with tempdir():
 
         mgr = mp.Manager()
 
@@ -65,7 +65,7 @@ def test_processData():
                                          vartable,
                                          proctable,
                                          cattable,
-                                         pool=pool,
+                                         njobs=mp.cpu_count(),
                                          mgr=mgr)
 
         dtable.proctable['Variable'] = [('all_independent', []),
@@ -78,7 +78,7 @@ def test_processData():
                                          vartable,
                                          proctable,
                                          cattable,
-                                         pool=pool,
+                                         njobs=mp.cpu_count(),
                                          mgr=mgr)
         dtable.proctable['Variable'] = [('vids', [2, 3]),
                                         ('vids', [1, 2])]
@@ -623,7 +623,6 @@ def test_processing_auxillaryVariables_parallel():
         open('processing.tsv', 'wt').write(procfile)
         gen_test_data(4, 50, 'data.tsv')
 
-        pool      = mp.Pool()
         mgr       = mp.Manager()
 
         proctable = loadtables.loadProcessingTable('processing.tsv')
@@ -632,7 +631,7 @@ def test_processing_auxillaryVariables_parallel():
                                          vartable,
                                          proctable,
                                          cattable,
-                                         pool=pool,
+                                         njobs=mp.cpu_count(),
                                          mgr=mgr)
 
         processing.processData(dtable)
