@@ -9,6 +9,7 @@ import string
 import random
 import datetime
 import itertools as it
+import functools as ft
 
 import pytest
 
@@ -286,19 +287,11 @@ def test_isSparse_date():
     assert core.isSparse(data, minpres=91) == (True,  'minpres', 90)
 
 
-
-def test_redundantColumns():
-
+def test_naCorrelation():
     size    = 50
     series1 = np.sin(np.linspace(0, np.pi * 6, size))
     series2 = series1 + np.random.random(size)
-    corr = pd.Series(series1).corr(pd.Series(series2))
-
-    data = pd.DataFrame({0 : pd.Series(series1), 1 : pd.Series(series2)})
-    cols = [(0, 1)]
-
-    assert core.redundantColumns(data, cols, corr * 1.01) == []
-    assert core.redundantColumns(data, cols, corr * 0.99) == [1]
+    series3 = series1 + np.random.random(size)
 
     # insert some missing values, making
     # sure there are more missing values
@@ -316,34 +309,70 @@ def test_redundantColumns():
     series1[s1miss] = np.nan
     series2[s2miss] = np.nan
 
-    corr   = pd.Series(series1).corr(pd.Series(series2))
     nacorr = np.corrcoef(np.isnan(series1), np.isnan(series2))[0, 1]
 
+    def check(df, nathres, exp):
+        got = core.naCorrelation(df.isna(), nathres)
+        exp = np.array(exp)
+        assert np.all(np.isclose(got, exp))
+
+    df = pd.DataFrame({0 : series1, 1 : series2})
+    check(df, nacorr * 1.01, np.eye(2))
+    check(df, nacorr * 0.99, [[1, 1], [1, 1]])
+
+    # cols with no missing values
+    # should always pass the test
+    df = pd.DataFrame({0 : series1, 1 : series2, 2 : series3})
+    check(df, nacorr * 1.01, [[1, 0, 1],
+                              [0, 1, 1],
+                              [1, 1, 1]])
+    check(df, nacorr * 0.99, [[1, 1, 1],
+                              [1, 1, 1],
+                              [1, 1, 1]])
+
+
+def wrapped_pairwiseRedundantColumns(df, corrthres):
+    xs, ys   = np.triu_indices(len(df.columns), k=1)
+    colpairs = np.vstack((xs, ys)).T
+    return core.pairwiseRedundantColumns(df, colpairs, corrthres)
+
+def test_pairwiseRedundantColumns():
+    _test_redundantColumns(wrapped_pairwiseRedundantColumns)
+
+def test_matrixRedundantColumns():
+    _test_redundantColumns(core.matrixRedundantColumns)
+
+def _test_redundantColumns(rcfunc):
+
+    size    = 50
+    series1 = np.sin(np.linspace(0, np.pi * 6, size))
+    series2 = series1 + np.random.random(size)
+    corr = pd.Series(series1).corr(pd.Series(series2))
+
     data = pd.DataFrame({0 : pd.Series(series1), 1 : pd.Series(series2)})
 
-    assert core.redundantColumns(data, cols, corr * 1.01) == []
-    assert core.redundantColumns(data, cols, corr * 0.99) == [1]
-
-    # both present and missing values must
-    # be above the threshold for the column
-    # to be considered redundant
-    assert core.redundantColumns(data, cols, corr * 0.99, nacorr * 0.99) == [1]
-    assert core.redundantColumns(data, cols, corr * 1.01, nacorr * 0.99) == []
-    assert core.redundantColumns(data, cols, corr * 0.99, nacorr * 1.01) == []
-    assert core.redundantColumns(data, cols, corr * 1.01, nacorr * 1.01) == []
+    assert np.all(rcfunc(data, corr * 1.01) == [])
+    assert np.all(rcfunc(data, corr * 0.99) == [1])
 
     # the column with more missing values
     # should be the one flagged as redundant
-    data = pd.DataFrame({0 : pd.Series(series2), 1 : pd.Series(series1)})
-    assert core.redundantColumns(data, cols, corr * 1.01)                == []
-    assert core.redundantColumns(data, cols, corr * 0.99)                == [0]
-    assert core.redundantColumns(data, cols, corr * 0.99, nacorr * 0.99) == [0]
-    assert core.redundantColumns(data, cols, corr * 1.01, nacorr * 0.99) == []
-    assert core.redundantColumns(data, cols, corr * 0.99, nacorr * 1.01) == []
-    assert core.redundantColumns(data, cols, corr * 1.01, nacorr * 1.01) == []
+    data.loc[0, 0] = np.nan
+    corr = pd.Series(data[0]).corr(data[1])
+    assert np.all(rcfunc(data, corr * 1.01) == [])
+    assert np.all(rcfunc(data, corr * 0.99) == [0])
+    data.loc[0:2, 1] = np.nan
+    corr = pd.Series(data[0]).corr(data[1])
+    assert np.all(rcfunc(data, corr * 1.01) == [])
+    assert np.all(rcfunc(data, corr * 0.99) == [1])
 
 
-def test_redundantColumns_associative():
+def test_pairwiseRedundantColumns_associative():
+    _test_redundantColumns_associative(wrapped_pairwiseRedundantColumns)
+
+def test_matrixRedundantColumns_associative():
+    _test_redundantColumns_associative(core.matrixRedundantColumns)
+
+def _test_redundantColumns_associative(rcfunc):
     a        = np.random.randint(1, 100, 100).astype(np.float32)
     b        = np.copy(a)
     c        = np.copy(a)
@@ -357,7 +386,7 @@ def test_redundantColumns_associative():
     # and b is always selected over c
     b[-1]    = np.nan
     c[-2:]   = np.nan
-    df       = pd.DataFrame({'a' : a, 'b' : b, 'c' : c})
+    df       = pd.DataFrame(np.array([a, b, c]).T, columns=['a', 'b', 'c'])
 
     corrmat  = df.corr()
     abthres  = corrmat.loc['a', 'b']
@@ -366,17 +395,100 @@ def test_redundantColumns_associative():
     # result should be the same regardless of column order
     for colorder in it.permutations('abc', 3):
         df = df[list(colorder)]
-        colpairs = list(it.combinations(df.columns, 2))
 
-        assert        core.redundantColumns(df, colpairs,  abthres * 1.05)         == []
-        assert        core.redundantColumns(df, colpairs, (abthres + bcthres) / 2) == ['b']
-        assert sorted(core.redundantColumns(df, colpairs,  bcthres * 0.95))        == ['b', 'c']
+        a = colorder.index('a')
+        b = colorder.index('b')
+        c = colorder.index('c')
+
+        assert        np.all(rcfunc(df,  abthres * 1.05)         == [])
+        assert        np.all(rcfunc(df, (abthres + bcthres) / 2) == [b])
+        assert np.all(sorted(rcfunc(df,  bcthres * 0.95))        == sorted([b, c]))
+
+
+# test that pairwise/matrix implementations produce same result
+def test_redundantColumns_na():
+    for _ in range(10):
+        npts, ncols      = 100, 100
+        hcols            = ncols // 2
+        seed             = np.random.randint(1, 10000000)
+        np.random.seed(seed)
+
+        data             = np.random.randn(npts, ncols)
+        data[:, :hcols] += data[:, hcols:]
+        naprops          = np.random.random(ncols) * 0.75
+        namask           = np.random.random((npts, ncols)) < naprops
+        data[namask]     = np.nan
+
+        df = pd.DataFrame(data)
+
+        nacorr    = np.corrcoef(namask.T)
+        dcorr     = df.corr().to_numpy()
+        nathres   = np.abs(nacorr[~np.isclose(nacorr, 1)]).mean()
+        corrthres = np.abs(dcorr[ ~np.isclose(dcorr,  1)]).mean()
+
+        nacorr   = core.naCorrelation(pd.isna(df), nathres)
+        colpairs = np.where(np.triu(nacorr, k=1))
+        colpairs = np.vstack(colpairs).T
+
+        pairwise = core.pairwiseRedundantColumns(df, colpairs, corrthres)
+        matrix   = core.matrixRedundantColumns(  df, corrthres, nathres)
+
+        nacounts = np.isnan(data).sum(axis=0)
+        corrmask = nacorr & (np.abs(dcorr) > corrthres)
+        colpairs = np.where(np.triu(corrmask, k=1))
+        colpairs = np.vstack(colpairs).T
+        exp = []
+        for ci, cj in colpairs:
+            if nacounts[ci] > nacounts[cj]: exp.append(ci)
+            else:                           exp.append(cj)
+
+        exp = np.unique(exp)
+
+        assert np.all(sorted(exp)      == sorted(matrix))
+        assert np.all(sorted(pairwise) == sorted(matrix))
+
+
+def test_redundantColumns_constant_allna():
+
+    seed = np.random.randint(1, 10000000)
+    np.random.seed(seed)
+
+    data       = np.random.randint(1, 100, (20, 10)).astype(np.float32)
+    data[0, :] = 20
+    data[1, :] = 30
+    data[2, :] = 40
+    data[3, :] = 50
+    data[4, :] = np.nan
+    data[5, :] = np.nan
+    data[6, :] = np.nan
+    data[7, :] = np.nan
+    data[0, 0] = np.nan
+    data[1, 1] = np.nan
+    data[4, 2] = 7
+    data[5, 3] = 8
+    data[8, 0] = np.nan
+    data[9, 1] = np.nan
+    df         = pd.DataFrame(data.T)
+
+    nacorr   = core.naCorrelation(pd.isna(df), 0.1)
+    colpairs = np.where(np.triu(nacorr, k=1))
+    colpairs = np.vstack(colpairs).T
+
+    pairwise = core.pairwiseRedundantColumns(df, colpairs, 0.2)
+    matrix   = core.matrixRedundantColumns(  df, 0.2, 0.1)
+
+    pairwise = np.array(sorted(pairwise))
+    matrix   = np.array(sorted(matrix))
+    assert np.all(pairwise == matrix)
 
 
 def test_binariseCategorical():
 
-    data = np.random.randint(1, 10, (100, 10))
-    cols = {str(i + 1) : data[:, i] for i in range(10)}
+    nvals = 100
+    ncols = 100
+    nrows = 1000
+    data = np.random.randint(1, nvals + 1, (nrows, ncols))
+    cols = {str(i + 1) : data[:, i] for i in range(ncols)}
     df   = pd.DataFrame(cols)
 
     bindata, uniq = core.binariseCategorical(df)
@@ -389,12 +501,12 @@ def test_binariseCategorical():
 
     data[data == 5] = 6
     data[:10, 0] = 5
-    cols = {str(i + 1) : data[:, i] for i in range(10)}
+    cols = {str(i + 1) : data[:, i] for i in range(ncols)}
     df   = pd.DataFrame(cols)
 
     bindata, uniq = core.binariseCategorical(df, minpres=11)
 
-    assert sorted(uniq) == [1, 2, 3, 4, 6, 7, 8, 9]
+    assert sorted(uniq) == sorted(set(range(1, nvals + 1)).difference([5]))
 
     for i, v in enumerate(uniq):
         exp = np.any(data == v, axis=1)
@@ -425,12 +537,17 @@ def test_binariseCategorical_missing():
 
 
 def test_binariseCategorical_take():
-    data = np.full((10, 5), np.nan)
-    aux  = np.full((10, 5), np.nan)
+
+    nvals = 100
+    ncols = 100
+    nrows = 1000
+
+    data = np.full((nrows, ncols), np.nan)
+    aux  = np.full((nrows, ncols), np.nan)
 
     for i in range(data.shape[1]):
         namask = np.random.random(data.shape[0]) < 0.7
-        data[~namask, i] = np.random.randint(1, 10, (~namask).sum())
+        data[~namask, i] = np.random.randint(1, nvals + 1, (~namask).sum())
 
     mask      = np.isfinite(data)
     aux[mask] = np.random.randint(1, 100, data.shape)[mask]
@@ -463,7 +580,6 @@ def test_binariseCategorical_take():
     assert np.all(gotdata[~gotna] == expdata[~expna])
 
 
-
 def test_binariseCategorical_take_error():
 
     data = pd.DataFrame(np.random.randint(1, 10, (100, 10)))
@@ -473,6 +589,69 @@ def test_binariseCategorical_take_error():
         core.binariseCategorical(data, take=take)
 
 
+def test_binariseCategorical_parallel():
+
+    nrows = 10000
+    ncols = 100
+    nvals = 500
+    data = np.random.randint(1, nvals + 1, (nrows, ncols))
+    cols = {str(i + 1) : data[:, i] for i in range(ncols)}
+    df   = pd.DataFrame(cols)
+
+    seqbindata, sequniq = core.binariseCategorical(df)
+    parbindata, paruniq = core.binariseCategorical(df, njobs=6)
+
+    assert np.all(seqbindata == parbindata)
+    assert np.all(sequniq == paruniq)
+
+    aux  = np.random.randint(nvals, nvals + 500, (nrows, ncols))
+    cols = {str(i + 1) : aux[:, i] for i in range(aux.shape[1])}
+    adf  = pd.DataFrame(cols)
+
+    seqbindata, sequniq = core.binariseCategorical(df, take=adf)
+    parbindata, paruniq = core.binariseCategorical(df, take=adf, njobs=6)
+
+    assert np.all(seqbindata == parbindata)
+    assert np.all(sequniq == paruniq)
+
+
+def test_binariseCategorical_datetime():
+    nrows = 5
+    ncols = 10
+    nvals = 10
+    data = np.random.randint(1, nvals + 1, (nrows, ncols))
+    cols = {str(i + 1) : data[:, i] for i in range(ncols)}
+    df   = pd.DataFrame(cols)
+
+    aux  = np.random.randint(100, 1000000, (nrows, ncols),
+                             dtype=np.uint64).astype('<M8[s]')
+    cols = {str(i + 1) : aux[:, i] for i in range(ncols)}
+    adf  = pd.DataFrame(cols)
+
+    seqbindata, sequniq = core.binariseCategorical(df, take=adf)
+    parbindata, paruniq = core.binariseCategorical(df, take=adf, njobs=6)
+
+    expuniq = list(sorted(np.unique(data)))
+    expdata = np.full((data.shape[0], len(expuniq)), np.nan, '<M8[s]')
+
+    for i, v in enumerate(expuniq):
+        rows, cols = np.where(data == v)
+        for r in range(data.shape[0]):
+            if r not in rows:
+                continue
+            c = cols[np.where(rows == r)[0][0]]
+            expdata[r, i] = aux[r, c]
+
+    expnan = np.isnan(expdata)
+
+    assert np.all(sequniq              == expuniq)
+    assert np.all(paruniq              == expuniq)
+    assert np.all(np.isnan(seqbindata) == expnan)
+    assert np.all(np.isnan(parbindata) == expnan)
+    assert np.all(expdata[~expnan]     == seqbindata[~expnan])
+    assert np.all(expdata[~expnan]     == parbindata[~expnan])
+
+
 def test_expandCompound():
 
     data = []
